# -*- coding: utf-8 -*-
from collective.plausible import _
from collective.plausible.interfaces import ICollectivePlausibleLayer
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.restapi.controlpanels import RegistryConfigletPanel
from plone.z3cform import layout
from zope import schema
from zope.component import adapter
from zope.interface import Interface


class IPlausibleControlPanel(Interface):
    url = schema.TextLine(
        title=_("Plausible URL"),
        description=_("Example : plausible.imio.be"),
        default="",
        required=False,
        readonly=False,
    )

    site = schema.TextLine(
        title=_("Plausible Site"),
        description=_("Example : imio.be"),
        default="",
        required=False,
        readonly=False,
    )

    token = schema.TextLine(
        title=_("Plausible token"),
        description=_("Plausible authentification token"),
        default="",
        required=False,
        readonly=False,
    )

    link_user_action = schema.Bool(
        title=_("Add a link in the user menu"),
        description=_("Add a link to the statistics browser view in the user menu"),
        default=True,
        required=False,
        readonly=False,
    )


class PlausibleControlPanel(RegistryEditForm):
    schema = IPlausibleControlPanel
    schema_prefix = "collective.plausible"
    label = _("Plausible Control Panel")


PlausibleControlPanelView = layout.wrap_form(
    PlausibleControlPanel, ControlPanelFormWrapper
)


@adapter(Interface, ICollectivePlausibleLayer)
class PlausibleControlPanelConfigletPanel(RegistryConfigletPanel):
    """Control Panel endpoint"""

    schema = IPlausibleControlPanel
    configlet_id = "plausible_control_panel-controlpanel"
    configlet_category_id = "Products"
    title = _("Plausible Control Panel")
    group = ""
    schema_prefix = "collective.plausible.plausible_control_panel"



# -*- coding: utf-8 -*-
from collective.plausible.testing import COLLECTIVE_PLAUSIBLE_FUNCTIONAL_TESTING
from collective.plausible.testing import COLLECTIVE_PLAUSIBLE_INTEGRATION_TESTING
from collective.plausible.views.plausible_view import IPlausibleView
from collective.plausible.views.plausible_view import PlausibleView
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest import mock
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface.interfaces import ComponentLookupError

import os
import unittest


class ViewsIntegrationTest(unittest.TestCase):

    layer = COLLECTIVE_PLAUSIBLE_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        api.content.create(self.portal, "Folder", "other-folder")
        api.content.create(self.portal, "Document", "front-page")

    def test_plausible_view_is_registered(self):
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="plausible-view"
        )
        self.assertTrue(IPlausibleView.providedBy(view))

    def test_plausible_view_not_matching_interface(self):
        view_found = True
        try:
            view = getMultiAdapter(
                (self.portal["front-page"], self.portal.REQUEST), name="plausible-view"
            )
        except ComponentLookupError:
            view_found = False
        else:
            view_found = IPlausibleView.providedBy(view)
        self.assertFalse(view_found)

    def set_registry_records(self):
        api.portal.set_registry_record("collective.plausible.site", "site-registry.be")
        api.portal.set_registry_record("collective.plausible.token", "token-registry")
        api.portal.set_registry_record("collective.plausible.url", "url-registry.be")

    @mock.patch.dict(
        os.environ,
        {
            "COLLECTIVE_PLAUSIBLE_SITE": "",
            "COLLECTIVE_PLAUSIBLE_TOKEN": "",
            "COLLECTIVE_PLAUSIBLE_URL": "",
        },
    )
    def test_noenv(self):
        view = PlausibleView(self.portal, self.request)
        self.assertFalse(view.is_plausible_set)
        self.set_registry_records()
        self.assertTrue(view.is_plausible_set)
        self.assertEqual(
            view.get_iframe_src,
            "https://url-registry.be/share/site-registry.be?auth=token-registry&embed=true&theme=light&background=transparent",
        )
        self.assertEqual(
            view.get_embedhostjs_src,
            "https://url-registry.be/js/embed.host.js",
        )

    @mock.patch.dict(
        os.environ,
        {
            "COLLECTIVE_PLAUSIBLE_SITE": "site-varenv.be",
            "COLLECTIVE_PLAUSIBLE_TOKEN": "token-varenv",
            "COLLECTIVE_PLAUSIBLE_URL": "url-varenv.be",
        },
    )
    def test_env(self):
        view = PlausibleView(self.portal, self.request)
        self.assertTrue(view.is_plausible_set)
        self.assertEqual(
            view.get_iframe_src,
            "https://url-varenv.be/share/site-varenv.be?auth=token-varenv&embed=true&theme=light&background=transparent",
        )
        self.assertEqual(
            view.get_embedhostjs_src,
            "https://url-varenv.be/js/embed.host.js",
        )
        self.set_registry_records()
        self.assertEqual(
            view.get_iframe_src,
            "https://url-varenv.be/share/site-varenv.be?auth=token-varenv&embed=true&theme=light&background=transparent",
        )
        self.assertEqual(
            view.get_embedhostjs_src,
            "https://url-varenv.be/js/embed.host.js",
        )

    def test_plausible_view(self):
        view = queryMultiAdapter((self.portal, self.request), name="plausible-view")
        self.assertNotIn("iframe", view())
        self.assertIn("Plausible analytics is not set", view())
        self.set_registry_records()
        self.assertIn("iframe", view())
        self.assertNotIn("Plausible analytics is not set", view())


class ViewsFunctionalTest(unittest.TestCase):

    layer = COLLECTIVE_PLAUSIBLE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])



# -*- coding: utf-8 -*-

from . import logger


from .base import reload_gs_profile
from collective.plausible import _
from plone.registry import Record, field
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def upgrade(setup_tool=None):
    """ """
    logger.info("Running upgrade (Python): Add action linking to @@plausible-view")
    registry = getUtility(IRegistry)
    if "collective.plausible.link_user_action" not in registry.records:
        registry_field = field.Bool(
            title=_("Add a link in the user menu"),
            description=_("Add a link to the statistics browser view in the user menu"),
            default=True,
            required=False,
            readonly=False,
        )
        registry_record = Record(registry_field)
        registry.records["collective.plausible.link_user_action"] = registry_record
    reload_gs_profile(setup_tool)



# -*- coding: utf-8 -*-

from plone import api

import os


def get_plausible_vars():
    env_plausible_url = os.getenv("COLLECTIVE_PLAUSIBLE_URL", "")
    env_plausible_site = os.getenv("COLLECTIVE_PLAUSIBLE_SITE", "")
    env_plausible_token = os.getenv("COLLECTIVE_PLAUSIBLE_TOKEN", "")

    plausible_url = (
        env_plausible_url
        if (env_plausible_url and env_plausible_url != "")
        else api.portal.get_registry_record("collective.plausible.url")
    )
    plausible_site = (
        env_plausible_site
        if (env_plausible_site and env_plausible_site != "")
        else api.portal.get_registry_record("collective.plausible.site")
    )
    plausible_token = (
        env_plausible_token
        if (env_plausible_token and env_plausible_token != "")
        else api.portal.get_registry_record("collective.plausible.token")
    )
    if all([plausible_site, plausible_url, plausible_token]):
        plausible_vars = {
            "plausible_url": plausible_url,
            "plausible_site": plausible_site,
            "plausible_token": plausible_token,
        }
        return plausible_vars
    else:
        return None



# -*- coding: utf-8 -*-

from collective.plausible.utils import get_plausible_vars
from Products.Five.browser import BrowserView
from zope.interface import implementer
from zope.interface import Interface


# from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class IPlausibleView(Interface):
    """Marker Interface for IPlausibleView"""


@implementer(IPlausibleView)
class PlausibleView(BrowserView):
    # If you want to define a template here, please remove the template from
    # the configure.zcml registration of this view.
    # template = ViewPageTemplateFile('plausible_view.pt')

    def __call__(self):
        # Implement your own actions:
        return self.index()

    @property
    def is_plausible_set(self):
        return True if get_plausible_vars() else False

    @property
    def get_embedhostjs_src(self):
        vars = get_plausible_vars()
        return f"https://{vars['plausible_url']}/js/embed.host.js"

    @property
    def get_iframe_src(self):
        vars = get_plausible_vars()
        return f"https://{vars['plausible_url']}/share/{vars['plausible_site']}?auth={vars['plausible_token']}&embed=true&theme=light&background=transparent"



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.CMFCore.utils import getToolByName

# from collective.plausible.testing import COLLECTIVE_PLAUSIBLE_FUNCTIONAL_TESTING
from collective.plausible.testing import COLLECTIVE_PLAUSIBLE_INTEGRATION_TESTING

import unittest


class UpgradeStepIntegrationTest(unittest.TestCase):

    layer = COLLECTIVE_PLAUSIBLE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_upgrade_step(self):
        portal_setup = getToolByName(self.portal, "portal_setup")
        portal_setup.runAllImportStepsFromProfile("collective.plausible:default")
        actions_tool = getToolByName(self.portal, "portal_actions")
        action_ids = actions_tool["user"].objectIds()
        self.assertIn("statistics", action_ids)


# class UpgradeStepFunctionalTest(unittest.TestCase):
#
#     layer = COLLECTIVE_PLAUSIBLE_FUNCTIONAL_TESTING
#
#     def setUp(self):
#         self.portal = self.layer['portal']
#         setRoles(self.portal, TEST_USER_ID, ['Manager'])



# -*- coding: utf-8 -*-

from collective.plausible.utils import get_plausible_vars
from zope.interface import Interface
from plone import api
from Products.Five.browser import BrowserView
from zope.interface import implementer


# from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class IPlausibleUtilsView(Interface):
    """Marker Interface for IPlausibleUtilsView"""


@implementer(IPlausibleUtilsView)
class PlausibleUtilsView(BrowserView):

    def is_plausible_set(self):
        return True if get_plausible_vars() else False

    def add_link_user_action(self):
        return (
            True
            if api.portal.get_registry_record(
                name="collective.plausible.link_user_action"
            )
            else False
        )
